
package regression;

/**
 * Context-free function grammar.
 *
 * @author	Copyright (C) 2002 Alessandro Bollini
 */
public class Grammar {

	private final Symbol root;


	public Grammar() {

		final Symbol variable=new Symbol();
		variable.setRules(new Object[][]{
			{"x"}
		});

		final Symbol operator=new Symbol();
		operator.setRules(new Object[][]{
			{"+"},
			{"-"},
			{"*"},
			{"/"}
		});

		final Symbol expression=new Symbol();
		expression.setRules(new Object[][]{
			{variable},
			{"(", expression, operator, expression, ")"}
		});

		this.root=expression;
	}


	public Symbol getRoot() {
		return root;
	}


	static public class Symbol {

		private Object[][] rules;


		public Symbol() {
			this(new Object[][]{});
		}

		public Symbol(final Object[][] rules) {
			this.rules=rules;
		}


		public void setRules(final Object[][] rules) {

			if ( rules == null ) {
				throw new IllegalArgumentException("null rule array");
			}

			this.rules=rules;
		}

		public int getOptions() {
			return rules.length;
		}

		public Object[] getRule(final int option) {
			return rules[option];
		}
	}

}
