
package knapsack;

import java.util.Vector;


/**
 * Knapsack.
 *
 * @author	Copyright (C) 2002 Alessandro Bollini
 */
public class Knapsack {

	private final int weightLimit;
	private final int volumeLimit;

	private Vector items=new Vector();

	private int weight=0;
	private int volume=0;
	private int value=0;


	public Knapsack(final int weightLimit, final int volumeLimit) {

		if ( weightLimit <= 0 ) {
			throw new IllegalArgumentException(
				"negative or null weight limit");
		}

		if ( volumeLimit <= 0 ) {
			throw new IllegalArgumentException(
				"negative or null volume limit");
		}

		this.weightLimit=weightLimit;
		this.volumeLimit=volumeLimit;
	}


	public int getWeight() {
		return weight;
	}

	public int getVolume() {
		return volume;
	}

	public int getValue() {
		return value;
	}


	public void add(final Item item)
		throws IllegalStateException {

		if ( item == null ) {
			throw new IllegalArgumentException("null item");
		}

		if ( weight+item.getWeight() > weightLimit ) {
			throw new IllegalStateException("weight limit exceeded");
		}

		if ( volume+item.getVolume() > volumeLimit ) {
			throw new IllegalStateException("volume limit exceeded");
		}

		items.add(item);

		weight+=item.getWeight();
		volume+=item.getVolume();
		value+=item.getValue();
	}

	public void empty() {

		items.clear();

		weight=0;
		volume=0;
		value=0;
	}


	public String toString() {

		final StringBuffer buffer=new StringBuffer();

		buffer.append("item\tweight\tvolume\tvalue\n");

		for ( int n=0; n < items.size(); ++n ) {

			final Item item=(Item)items.get(n);

			buffer.append("#"+(n+1));
			buffer.append("\t"+item.getWeight());
			buffer.append("\t"+item.getVolume());
			buffer.append("\t"+item.getValue());
			buffer.append("\n");
		}

		buffer.append("total");
		buffer.append("\t"+weight);
		buffer.append("\t"+volume);
		buffer.append("\t"+value);

		return buffer.toString();
	}
}
